package yrpg.application.crmvisit.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.botp.runtime.ConvertOperationResult;
import kd.bos.entity.botp.runtime.PushArgs;
import kd.bos.entity.botp.runtime.SourceBillReport;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.exception.KDBizException;
import kd.bos.servicehelper.botp.ConvertServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 单据操作插件
 * 专家的基础资料库: 审核后生成专家库
 */
public class ExpertBaseOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        String opKey = e.getOperationKey();
        super.endOperationTransaction(e);
        for (DynamicObject data : e.getDataEntities()) {

            if ("audit".equals(opKey)) {
                this.expertPush(data);
            }
        }
    }

    private void expertPush(DynamicObject data) {
        // 创建下推参数
        PushArgs pushArgs = new PushArgs();
        // 源单标识
        pushArgs.setSourceEntityNumber("yrpg_yzj_expert_base");
        // 目标单据标识
        pushArgs.setTargetEntityNumber("yrpg_crm_expert");
        // 生成转换结果报告
        pushArgs.setBuildConvReport(true);
        // 不检查目标单新增权限,非必填
        pushArgs.setHasRight(false);
        // 传入下推使用的转换规则id，不填则使用默认规则
//        pushArgs.setRuleId("1875512721336702976");
        // 下推默认保存
        pushArgs.setAutoSave(true);
        // 设置源单选中的数据包
        List<ListSelectedRow> selectedRows = new ArrayList<>();
        long pk = data.getLong("id");
        ListSelectedRow selectedRow = new ListSelectedRow(pk);
        selectedRows.add(selectedRow);
        pushArgs.setSelectedRows(selectedRows);
        // 执行下推操
        ConvertOperationResult result = ConvertServiceHelper.push(pushArgs);
        if (!result.isSuccess()) {
            for (SourceBillReport billReport : result.getBillReports()) {
                String failMessage = billReport.getFailMessage();
                throw new KDBizException("专家的基础资料库：" + failMessage);
            }
        } else {
            Set<Object> ids = result.getTargetBillIds();
            if (ids.size()>0) {
                //调用提交
                OperationServiceHelper.executeOperate("submit", "yrpg_crm_expert", ids.stream().toArray());
                //调用审核
                OperationServiceHelper.executeOperate("audit", "yrpg_crm_expert", ids.stream().toArray());

            }
        }
    }
}